@@ -18,7 +18,7 @@
  */
 
 /***************************************************************************
- * Copyright (C) 2017-2023 ZmartZone Holding BV
+ * Copyright (C) 2017-2024 ZmartZone Holding BV
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -44,15 +44,13 @@
  */
 
 #include "mod_auth_openidc.h"
+
+#include "metrics.h"
 #include "pcre_subst.h"
 
-#ifdef USE_LIBJQ
-#include "jq.h"
-#endif
-
-static apr_byte_t oidc_authz_match_value(request_rec *r, const char *spec_c,
-		json_t *val, const char *key) {
+static apr_byte_t oidc_authz_match_value(request_rec *r, const char *spec_c, const json_t *val, const char *key) {
 
+	const json_t *elem = NULL;
 	int i = 0;
 
 	oidc_debug(r, "matching: spec_c=%s, key=%s", spec_c, key);
@@ -60,19 +58,19 @@ static apr_byte_t oidc_authz_match_value(request_rec *r, const char *spec_c,
 	/* see if it is a string and it (case-insensitively) matches the Require'd value */
 	if (json_is_string(val)) {
 
-		if (apr_strnatcmp(json_string_value(val), spec_c) == 0)
+		if (_oidc_strcmp(json_string_value(val), spec_c) == 0)
 			return TRUE;
 
 		/* see if it is a integer and it equals the Require'd value */
 	} else if (json_is_integer(val)) {
 
-		if (json_integer_value(val) == atoi(spec_c))
+		if (json_integer_value(val) == _oidc_str_to_int(spec_c))
 			return TRUE;
 
 		/* see if it is a boolean and it (case-insensitively) matches the Require'd value */
 	} else if (json_is_boolean(val)) {
 
-		if (apr_strnatcmp(json_is_true(val) ? "true" : "false", spec_c) == 0)
+		if (_oidc_strcmp((json_is_true(val) ? "true" : "false"), spec_c) == 0)
 			return TRUE;
 
 		/* if it is an array, we'll walk it */
@@ -81,7 +79,7 @@ static apr_byte_t oidc_authz_match_value(request_rec *r, const char *spec_c,
 		/* compare the claim values */
 		for (i = 0; i < json_array_size(val); i++) {
 
-			json_t *elem = json_array_get(val, i);
+			elem = json_array_get(val, i);
 
 			if (json_is_string(elem)) {
 				/*
@@ -89,38 +87,34 @@ static apr_byte_t oidc_authz_match_value(request_rec *r, const char *spec_c,
 				 * whitespace). At this point, spec_c points to the
 				 * NULL-terminated value pattern.
 				 */
-				if (apr_strnatcmp(json_string_value(elem), spec_c) == 0)
+				if (_oidc_strcmp(json_string_value(elem), spec_c) == 0)
 					return TRUE;
 
 			} else if (json_is_boolean(elem)) {
 
-				if (apr_strnatcmp(
-						json_is_true(elem) ? "true" : "false", spec_c) == 0)
+				if (_oidc_strcmp((json_is_true(elem) ? "true" : "false"), spec_c) == 0)
 					return TRUE;
 
 			} else if (json_is_integer(elem)) {
 
-				if (json_integer_value(elem) == atoi(spec_c))
+				if (json_integer_value(elem) == _oidc_str_to_int(spec_c))
 					return TRUE;
 
 			} else {
 
-				oidc_warn(r,
-						"unhandled in-array JSON object type [%d] for key \"%s\"",
-						elem->type, (const char * ) key);
+				oidc_warn(r, "unhandled in-array JSON object type [%d] for key \"%s\"", elem->type,
+					  (const char *)key);
 			}
-
 		}
 
 	} else {
-		oidc_warn(r, "unhandled JSON object type [%d] for key \"%s\"",
-				val->type, (const char * ) key);
+		oidc_warn(r, "unhandled JSON object type [%d] for key \"%s\"", val->type, (const char *)key);
 	}
 
 	return FALSE;
 }
 
-static apr_byte_t oidc_authz_match_expression(request_rec *r, const char *spec_c, json_t *val) {
+static apr_byte_t oidc_authz_match_expression(request_rec *r, const char *spec_c, const json_t *val) {
 	apr_byte_t rc = FALSE;
 	struct oidc_pcre *preg = NULL;
 	char *error_str = NULL;
@@ -139,8 +133,8 @@ static apr_byte_t oidc_authz_match_expression(request_rec *r, const char *spec_c
 
 		error_str = NULL;
 		/* PCRE-compare the string value against the expression */
-		if (oidc_pcre_exec(r->pool, preg, json_string_value(val), (int) strlen(json_string_value(val)), &error_str)
-				> 0) {
+		if (oidc_pcre_exec(r->pool, preg, json_string_value(val), (int)_oidc_strlen(json_string_value(val)),
+				   &error_str) > 0) {
 			oidc_debug(r, "value \"%s\" matched regex \"%s\"", json_string_value(val), spec_c);
 			rc = TRUE;
 			goto end;
@@ -159,9 +153,10 @@ static apr_byte_t oidc_authz_match_expression(request_rec *r, const char *spec_c
 
 				error_str = NULL;
 				/* PCRE-compare the string value against the expression */
-				if (oidc_pcre_exec(r->pool, preg, json_string_value(elem), (int) strlen(json_string_value(elem)), &error_str)
-						> 0) {
-					oidc_debug(r, "array value \"%s\" matched regex \"%s\"", json_string_value(elem), spec_c);
+				if (oidc_pcre_exec(r->pool, preg, json_string_value(elem),
+						   (int)_oidc_strlen(json_string_value(elem)), &error_str) > 0) {
+					oidc_debug(r, "array value \"%s\" matched regex \"%s\"",
+						   json_string_value(elem), spec_c);
 					rc = TRUE;
 					goto end;
 				} else if (error_str) {
@@ -183,8 +178,7 @@ end:
 /*
  * see if a the Require value matches with a set of provided claims
  */
-apr_byte_t oidc_authz_match_claim(request_rec *r, const char * const attr_spec,
-		const json_t * const claims) {
+apr_byte_t oidc_authz_match_claim(request_rec *r, const char *const attr_spec, json_t *claims) {
 
 	const char *key;
 	json_t *val;
@@ -194,15 +188,15 @@ apr_byte_t oidc_authz_match_claim(request_rec *r, const char * const attr_spec,
 		return FALSE;
 
 	/* loop over all of the user claims */
-	void *iter = json_object_iter((json_t*) claims);
+	void *iter = json_object_iter(claims);
 	while (iter) {
 
 		key = json_object_iter_key(iter);
 		val = json_object_iter_value(iter);
 
-		oidc_debug(r, "evaluating key \"%s\"", (const char * ) key);
+		oidc_debug(r, "evaluating key \"%s\"", (const char *)key);
 
-		const char *attr_c = (const char *) key;
+		const char *attr_c = key;
 		const char *spec_c = attr_spec;
 
 		/* walk both strings until we get to the end of either or we find a differing character */
@@ -236,27 +230,21 @@ apr_byte_t oidc_authz_match_claim(request_rec *r, const char * const attr_spec,
 			spec_c++;
 
 			if (json_is_object(val)) {
-				oidc_debug(r,
-						"attribute chunk matched, evaluating children of key: \"%s\".",
-						key);
-				return oidc_authz_match_claim(r, spec_c,
-						json_object_get(claims, key));
+				oidc_debug(r, "attribute chunk matched, evaluating children of key: \"%s\".", key);
+				return oidc_authz_match_claim(r, spec_c, json_object_get(claims, key));
 			} else if (json_is_array(val)) {
-				oidc_debug(r,
-						"attribute chunk matched, evaluating array values of key: \"%s\".",
-						key);
-				return oidc_authz_match_value(r, spec_c,
-						json_object_get(claims, key), key);
+				oidc_debug(r, "attribute chunk matched, evaluating array values of key: \"%s\".", key);
+				return oidc_authz_match_value(r, spec_c, json_object_get(claims, key), key);
 			} else {
 				oidc_warn(r,
-						"\"%s\" matched, and child nodes or array values should be evaluated, but value is not an object or array.",
-						key);
+					  "\"%s\" matched, and child nodes or array values should be evaluated, but "
+					  "value is not an object or array.",
+					  key);
 				return FALSE;
 			}
-
 		}
 
-		iter = json_object_iter_next((json_t *) claims, iter);
+		iter = json_object_iter_next(claims, iter);
 	}
 
 	return FALSE;
@@ -264,72 +252,42 @@ apr_byte_t oidc_authz_match_claim(request_rec *r, const char * const attr_spec,
 
 #ifdef USE_LIBJQ
 
-static apr_byte_t jq_parse(request_rec *r, jq_state *jq, struct jv_parser *parser) {
-	apr_byte_t rv = FALSE;
-	jv value;
-
-	while (jv_is_valid((value = jv_parser_next(parser)))) {
-		jq_start(jq, value, 0);
-		jv result;
-
-		while (jv_is_valid(result = jq_next(jq))) {
-			jv dumped = jv_dump_string(result, 0);
-			const char *str = jv_string_value(dumped);
-			oidc_debug(r, "dumped: %s", str);
-			rv = (apr_strnatcmp(str, "true") == 0);
-		}
-
-		jv_free(result);
-	}
-
-	if (jv_invalid_has_msg(jv_copy(value))) {
-		jv msg = jv_invalid_get_msg(value);
-		oidc_error(r, "invalid: %s", jv_string_value(msg));
-		jv_free(msg);
-		rv = FALSE;
-	} else {
-		jv_free(value);
-	}
-
-	return rv;
-}
-
 /*
  * see if a the Require value matches a configured expression
  */
-apr_byte_t oidc_authz_match_claims_expr(request_rec *r,
-		const char * const attr_spec, const json_t * const claims) {
+apr_byte_t oidc_authz_match_claims_expr(request_rec *r, const char *const attr_spec, json_t *claims) {
 	apr_byte_t rv = FALSE;
+	const char *str = NULL;
 
 	oidc_debug(r, "enter: '%s'", attr_spec);
 
-	jq_state *jq = jq_init();
-	if (jq_compile(jq, attr_spec) == 0) {
-		jq_teardown(&jq);
-		return FALSE;
-	}
-
-	struct jv_parser *parser = jv_parser_new(0);
-
-	char *buf = oidc_util_encode_json_object(r, (json_t *)claims, 0);
-	jv_parser_set_buf(parser, buf, strlen(buf), 0);
-	rv = jq_parse(r, jq, parser);
-
-	jv_parser_free(parser);
-	jq_teardown(&jq);
+	str = oidc_util_jq_filter(r, oidc_util_encode_json_object(r, claims, JSON_PRESERVE_ORDER | JSON_COMPACT),
+				  attr_spec);
+	rv = (_oidc_strcmp(str, "true") == 0);
 
 	return rv;
 }
 
 #endif
 
-#if MODULE_MAGIC_NUMBER_MAJOR < 20100714
+#define OIDC_AUTHZ_ERROR "OIDC_AUTHZ_ERROR"
+
+static void oidc_authz_error_add(request_rec *r, const char *msg) {
+	const char *envvar = NULL;
+	if (r->subprocess_env != NULL) {
+		envvar = apr_table_get(r->subprocess_env, OIDC_AUTHZ_ERROR);
+		oidc_debug(r, "adding %s to environment variable %s=%s", msg, OIDC_AUTHZ_ERROR, envvar);
+		apr_table_set(r->subprocess_env, OIDC_AUTHZ_ERROR,
+			      apr_psprintf(r->pool, "%s%s%s", envvar ? envvar : "", envvar ? "," : "", msg ? msg : ""));
+	}
+}
+
+#if !(HAVE_APACHE_24)
 
 /*
  * Apache <2.4 authorization routine: match the claims from the authenticated user against the Require primitive
  */
-int oidc_authz_worker22(request_rec *r, const json_t * const claims,
-		const require_line * const reqs, int nelts) {
+int oidc_authz_worker22(request_rec *r, json_t *claims, const require_line *const reqs, int nelts) {
 	const int m = r->method_number;
 	const char *token;
 	const char *requirement;
@@ -352,10 +310,10 @@ int oidc_authz_worker22(request_rec *r, const json_t * const claims,
 		token = ap_getword_white(r->pool, &requirement);
 
 		/* see if we've got anything meant for us */
-		if (apr_strnatcasecmp(token, OIDC_REQUIRE_CLAIM_NAME) == 0) {
+		if (_oidc_strnatcasecmp(token, OIDC_REQUIRE_CLAIM_NAME) == 0) {
 			match_claim_fn = oidc_authz_match_claim;
 #ifdef USE_LIBJQ
-		} else if (apr_strnatcasecmp(token, OIDC_REQUIRE_CLAIMS_EXPR_NAME) == 0) {
+		} else if (_oidc_strnatcasecmp(token, OIDC_REQUIRE_CLAIMS_EXPR_NAME) == 0) {
 			match_claim_fn = oidc_authz_match_claims_expr;
 #endif
 		} else {
@@ -386,11 +344,14 @@ int oidc_authz_worker22(request_rec *r, const json_t * const claims,
 
 			if (match_claim_fn(r, token, claims) == TRUE) {
 
-				/* if *any* claim matches, then authorization has succeeded and all of the others are ignored */
+				/* if *any* claim matches, then authorization has succeeded and all of the others are
+				 * ignored */
 				oidc_debug(r, "require claim/expr '%s' matched", token);
 				return OK;
 			}
 		}
+
+		oidc_authz_error_add(r, requirement);
 	}
 
 	/* if there weren't any "Require claim" directives, we're irrelevant */
@@ -398,15 +359,17 @@ int oidc_authz_worker22(request_rec *r, const json_t * const claims,
 		oidc_debug(r, "no claim/expr statements found, not performing authz");
 		return DECLINED;
 	}
-	/* if there was a "Require claim", but no actual claims, that's cause to warn the admin of an iffy configuration */
+	/* if there was a "Require claim", but no actual claims, that's cause to warn the admin of an iffy configuration
+	 */
 	if (count_oauth_claims == 0) {
-		oidc_warn(r,
-				"'require claim/expr' missing specification(s) in configuration, declining");
+		oidc_warn(r, "'require claim/expr' missing specification(s) in configuration, declining");
 		return DECLINED;
 	}
 
 	/* log the event, also in Apache speak */
-	oidc_info(r, "authorization denied for require claims (0/%d): '%s'", nelts, nelts > 0 ? reqs[0].requirement : "(none)");
+	oidc_debug(r, "authorization denied for require claims (0/%d): '%s'", nelts,
+		   nelts > 0 ? reqs[0].requirement : "(none)");
+
 	ap_note_auth_failure(r);
 
 	return HTTP_UNAUTHORIZED;
@@ -417,9 +380,10 @@ int oidc_authz_worker22(request_rec *r, const json_t * const claims,
 /*
  * Apache >=2.4 authorization routine: match the claims from the authenticated user against the Require primitive
  */
-authz_status oidc_authz_worker24(request_rec *r, const json_t * const claims,
-		const char *require_args, const void *parsed_require_args, oidc_authz_match_claim_fn_type match_claim_fn) {
+authz_status oidc_authz_worker24(request_rec *r, json_t *claims, const char *require_args,
+				 const void *parsed_require_args, oidc_authz_match_claim_fn_type match_claim_fn) {
 
+	oidc_cfg *cfg = ap_get_module_config(r->server->module_config, &auth_openidc_module);
 	int count_oauth_claims = 0;
 	const char *t, *w, *err = NULL;
 	const ap_expr_info_t *expr = parsed_require_args;
@@ -452,6 +416,8 @@ authz_status oidc_authz_worker24(request_rec *r, const json_t * const claims,
 		/* see if we can match any of out input claims against this Require'd value */
 		if (match_claim_fn(r, w, claims) == TRUE) {
 
+			OIDC_METRICS_COUNTER_INC_SPEC(r, cfg, OM_AUTHZ_MATCH_REQUIRE_CLAIM, require_args);
+
 			oidc_debug(r, "require claim/expr '%s' matched", w);
 			return AUTHZ_GRANTED;
 		}
@@ -459,11 +425,13 @@ authz_status oidc_authz_worker24(request_rec *r, const json_t * const claims,
 
 	/* if there wasn't anything after the Require claims directive... */
 	if (count_oauth_claims == 0) {
-		oidc_warn(r,
-				"'require claim/expr' missing specification(s) in configuration, denying");
+		oidc_warn(r, "'require claim/expr' missing specification(s) in configuration, denying");
 	}
 
-	oidc_info(r, "could not match require claim expression '%s'", require_args);
+	OIDC_METRICS_COUNTER_INC_SPEC(r, cfg, OM_AUTHZ_ERROR_REQUIRE_CLAIM, require_args);
+
+	oidc_debug(r, "could not match require claim expression '%s'", require_args);
+	oidc_authz_error_add(r, require_args);
 
 	return AUTHZ_DENIED;
 }
